@@ -1,4 +1,4 @@
-/* 
+/*
  *  playdv.c
  *
  *     Copyright (C) Charles 'Buck' Krasic - April 2000
@@ -8,23 +8,27 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
  */
 
-#include <libdv/dv_types.h>
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#define _FILE_OFFSET_BITS 64
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,7 +40,7 @@
 #include <fcntl.h>
 #include <string.h>
 
-#include <libdv/dv.h>
+#include "libdv/dv.h"
 #include "display.h"
 #include "oss.h"
 
@@ -62,7 +66,8 @@
 #define DV_PLAYER_OPT_AUTOHELP        7
 #define DV_PLAYER_OPT_DUMP_FRAMES     8
 #define DV_PLAYER_OPT_NO_MMAP         9
-#define DV_PLAYER_NUM_OPTS            10
+#define DV_PLAYER_OPT_LOOP_COUNT      10
+#define DV_PLAYER_NUM_OPTS            11
 
 /* Book-keeping for mmap */
 typedef struct dv_mmap_region_s {
@@ -83,6 +88,7 @@ typedef struct {
   gint             no_mmap;
   gint             arg_num_frames;
   char*            arg_dump_frames;
+  gint             arg_loop_count;
 #if HAVE_LIBPOPT
   struct poptOption option_table[DV_PLAYER_NUM_OPTS+1]; 
 #endif /* HAVE_LIBPOPT */
@@ -97,6 +103,7 @@ dv_player_new(void)
   if(!(result->display = dv_display_new())) goto no_display;
   if(!(result->oss = dv_oss_new())) goto no_oss;
   if(!(result->decoder = dv_decoder_new(TRUE, FALSE, FALSE))) goto no_decoder;
+  result->arg_loop_count = 1;
 
 #if HAVE_LIBPOPT
   result->option_table[DV_PLAYER_OPT_VERSION] = (struct poptOption) {
@@ -142,6 +149,15 @@ dv_player_new(void)
     descrip:    "don't use mmap for reading. (usefull for pipes)"
   }; /* no mmap */
 
+  result->option_table[DV_PLAYER_OPT_LOOP_COUNT] = (struct poptOption) {
+    longName:   "loop-count", 
+    shortName:  'l', 
+    argInfo:    POPT_ARG_INT, 
+    arg:        &result->arg_loop_count,
+    argDescrip: "count",
+    descrip:    "loop playback <count> times, 0 for infinite",
+  }; /* loop count */
+
   result->option_table[DV_PLAYER_OPT_OSS_INCLUDE] = (struct poptOption) {
     argInfo: POPT_ARG_INCLUDE_TABLE,
     arg:     result->oss->option_table,
@@ -294,7 +310,7 @@ main(int argc,char *argv[])
 #if HAVE_LIBPOPT
   /* Parse options using popt */
   optCon = poptGetContext(NULL, argc, (const char **)argv, dv_player->option_table, 0);
-  poptSetOtherOptionHelp(optCon, "<filename>");
+  poptSetOtherOptionHelp(optCon, "[raw dv file or -- for stdin]");
 
   while ((rc = poptGetNextOpt(optCon)) > 0) {
     switch (rc) {
@@ -306,10 +322,10 @@ main(int argc,char *argv[])
     } /* switch */
   } /* while */
 
-  if (rc < -1) goto bad_arg;
-
   filename = poptGetArg(optCon);
-  if((filename == NULL) || !(poptPeekArg(optCon) == NULL)) goto bad_filename;
+  if (rc < -1) goto bad_arg;
+  if((filename == NULL) || !(poptPeekArg(optCon) == NULL))
+	  filename = "-";
   poptFreeContext(optCon);
 #else
   /* No popt, no usage and no options!  HINT: get popt if you don't
@@ -362,7 +378,7 @@ main(int argc,char *argv[])
   } /* if */
 
   dv_player->arg_disable_audio = 
-    dv_player->arg_disable_audio || (!dv_oss_init(dv_player->decoder->audio, dv_player->oss));
+    dv_player->arg_disable_audio || (!dv_oss_init(dv_player->decoder, dv_player->oss));
 
   for(i=0; i < 4; i++) {
     if(!(audio_buffers[i] = malloc(DV_AUDIO_MAX_SAMPLES*sizeof(gint16)))) goto no_mem;
@@ -370,16 +386,18 @@ main(int argc,char *argv[])
 
   gettimeofday(dv_player->tv+0,NULL);
 
+restart:
+
   dv_player->decoder->prev_frame_decoded = 0;
   for(offset=0;
       offset <= eof || dv_player->no_mmap; 
       offset += dv_player->decoder->frame_size) {
 
-    /*
+     /*
      * Map the frame's data into memory
      */
     mmap_unaligned (fd,dv_player->no_mmap, 
-		    offset, dv_player->decoder->frame_size, 
+		    offset, dv_player->decoder->frame_size,
 		    &dv_player->mmap_region);
     if (MAP_FAILED == dv_player->mmap_region.map_start) goto map_failed;
     if (dv_parse_header (dv_player->decoder,
@@ -408,7 +426,7 @@ main(int argc,char *argv[])
     /* Parse and unshuffle audio */
     if(!dv_player->arg_disable_audio) {
       if(dv_decode_full_audio(dv_player->decoder, dv_player->mmap_region.data_start, audio_buffers)) {
-	dv_oss_play(dv_player->decoder->audio, dv_player->oss, audio_buffers);
+	dv_oss_play(dv_player->decoder, dv_player->oss, audio_buffers);
       } /* if */
     } /* if */
 
@@ -477,6 +495,13 @@ main(int argc,char *argv[])
   } /* while */
 
  end_of_file:
+  
+  /* Handle looping */
+  if (--dv_player->arg_loop_count != 0) {
+    lseek( fd, 0, SEEK_SET);
+    goto restart;
+  }
+  
   gettimeofday(dv_player->tv+1,NULL);
   timersub(dv_player->tv+1,dv_player->tv+0,dv_player->tv+2);
   seconds = (double)dv_player->tv[2].tv_usec / 1000000.0; 
@@ -506,10 +531,6 @@ main(int argc,char *argv[])
 	  poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
 	  poptStrerror(rc));
   exit(-1);
- bad_filename:
-  poptPrintUsage(optCon, stderr, 0);
-  fprintf(stderr, "\nSpecify a single <filename> argument; e.g. pond.dv\n");
-  exit(-1);
 #endif
  no_display:
   exit(-1);
